import { CustomResource, RemovalPolicy } from 'aws-cdk-lib';
import { ISecurityGroup } from 'aws-cdk-lib/aws-ec2';
import { IPrincipal, IRole } from 'aws-cdk-lib/aws-iam';
import { IFunction } from 'aws-cdk-lib/aws-lambda';
import { ILogGroup } from 'aws-cdk-lib/aws-logs';
import { Construct } from 'constructs';
import { KafkaApiProps } from './kafka-api-props';
import { Authentication, Acl, MskTopic } from './msk-utils';
import { TrackedConstruct } from '../../../utils';
/**
 * A construct to create an MSK Serverless cluster
 * @see https://awslabs.github.io/data-solutions-framework-on-aws/
 *
 * @example
 */
export declare class KafkaApi extends TrackedConstruct {
    /**
     * The IAM Role used by the Custom Resource provider when MSK is using IAM authentication
     */
    readonly mskIamRole?: IRole;
    /**
     * The Cloudwatch Log Group used by the Custom Resource provider when MSK is using IAM authentication
     */
    readonly mskIamLogGroup?: ILogGroup;
    /**
     * The Lambda function used by the Custom Resource provider when MSK is using IAM authentication
     */
    readonly mskIamFunction?: IFunction;
    /**
     * The Security Group used by the Custom Resource provider when MSK is using IAM authentication
     */
    readonly mskIamSecurityGroup?: ISecurityGroup[];
    /**
     * The IAM Role used by the Custom Resource provider when MSK is using mTLS authentication
     */
    readonly mskAclRole?: IRole;
    /**
     * The Cloudwatch Log Group used by the Custom Resource provider when MSK is using mTLS authentication
     */
    readonly mskAclLogGroup?: ILogGroup;
    /**
     * The Lambda function used by the Custom Resource provider when MSK is using mTLS authentication
     */
    readonly mskAclFunction?: IFunction;
    /**
     * The Security Group used by the Custom Resource provider when MSK is using mTLS authentication
     */
    readonly mskAclSecurityGroup?: ISecurityGroup[];
    private readonly mskAclServiceToken?;
    private readonly mskIamServiceToken?;
    private readonly removalPolicy;
    private readonly kafkaClientLogLevel;
    private readonly tlsCertifacateSecret?;
    private readonly clusterArn;
    private readonly clusterType;
    /**
     * Constructs a new instance of the EmrEksCluster construct.
     * @param {Construct} scope the Scope of the CDK Construct
     * @param {string} id the ID of the CDK Construct
     * @param {MskServerlessProps} props
     */
    constructor(scope: Construct, id: string, props: KafkaApiProps);
    /**
     * Creates a topic in the Msk Cluster
     * @param {string} id the CDK id for Topic
     * @param {Acl} aclDefinition the Kafka Acl definition
     * @param {RemovalPolicy} removalPolicy Wether to keep the ACL or delete it when removing the resource from the Stack. @default - RemovalPolicy.RETAIN
     * @returns {CustomResource} The MskAcl custom resource created
     */
    setAcl(id: string, aclDefinition: Acl, removalPolicy?: RemovalPolicy): CustomResource;
    /**
     * Creates a topic in the Msk Cluster
     * @param {string} id the CDK id for Topic
     * @param {Authentication} clientAuthentication The client authentication to use when creating the Topic
     * @param {MskTopic} topicDefinition the Kafka topic definition
     * @param {RemovalPolicy} removalPolicy Wether to keep the topic or delete it when removing the resource from the Stack. @default - RemovalPolicy.RETAIN
     * @param {boolean} waitForLeaders If this is true it will wait until metadata for the new topics doesn't throw LEADER_NOT_AVAILABLE
     * @param {number} timeout The time in ms to wait for a topic to be completely created on the controller node @default - 5000
     * @returns {CustomResource} The MskTopic custom resource created
     */
    setTopic(id: string, clientAuthentication: Authentication, topicDefinition: MskTopic, removalPolicy?: RemovalPolicy, waitForLeaders?: boolean, timeout?: number): CustomResource;
    /**
     * Grant a principal to produce data to a topic
     * @param {string} id the CDK resource id
     * @param {string} topicName the topic to which the principal can produce data
     * @param {Authentitcation} clientAuthentication The client authentication to use when grant on resource
     * @param {IPrincipal | string } principal the IAM principal to grand the produce to
     * @param {string} host the host to which the principal can produce data. @default - * is used
     * @param {RemovalPolicy} removalPolicy the removal policy to apply to the grant. @default - RETAIN is used
     * @returns When MTLS is used as authentication an ACL is created using the MskAcl Custom resource to write in the topic is created and returned,
     *          you can use it to add dependency on it.
     */
    grantProduce(id: string, topicName: string, clientAuthentication: Authentication, principal: IPrincipal | string, host?: string, removalPolicy?: RemovalPolicy): CustomResource | undefined;
    /**
     * Grant a principal the right to consume data from a topic
     * @param {string} id the CDK resource id
     * @param {string} topicName the topic to which the principal can produce data
     * @param {Authentitcation} clientAuthentication The client authentication to use when grant on resource
     * @param {IPrincipal | string } principal the IAM principal to grand the produce to
     * @param {string} host the host to which the principal can produce data. @default - * is used
     * @param {RemovalPolicy} removalPolicy the removal policy to apply to the grant. @default - RETAIN is used
     * @returns When MTLS is used as authentication an ACL is created using the MskAcl Custom resource to read from the topic is created and returned,
     *          you can use it to add dependency on it.
     */
    grantConsume(id: string, topicName: string, clientAuthentication: Authentication, principal: IPrincipal | string, host?: string, removalPolicy?: RemovalPolicy): CustomResource | undefined;
}
